from django.test import TestCase

from . import fixtures
from . import mocks

from djconnectwise.callbacks import CallbacksHandler


NEEDED_CALLBACKS = [
    {
        "type": "ticket",
        "description": "Kanban application ticket callback",
        "url": None,
        "objectId": 1,
        "level": "owner",
    },
    {
        "type": "project",
        "description": "Kanban application project callback",
        "url": None,
        "objectId": 1,
        "level": "owner",
    },
    {
        "type": "company",
        "description": "Kanban application company callback",
        "url": None,
        "objectId": 1,
        "level": "owner",
    }
]


class TestCallBackHandler(TestCase):
    def setUp(self):
        super().setUp()
        self.handler = CallbacksHandler()

    def test_calculate_no_changes(self):
        needed = NEEDED_CALLBACKS.copy()
        current = needed.copy()
        add, remove = self.handler._calculate_missing_unneeded_callbacks(
            needed, current
        )
        self.assertEqual(add, [])
        self.assertEqual(remove, [])

    def test_calculate_callback_needs_adding(self):
        needed = NEEDED_CALLBACKS.copy()
        current = [
            {
                "type": "ticket",
                "description": "Kanban application ticket callback",
                "url": None,
                "objectId": 1,
                "level": "owner",
            },
            {
                "type": "company",
                "description": "Kanban application company callback",
                "url": None,
                "objectId": 1,
                "level": "owner",
            }
        ]
        add, remove = self.handler._calculate_missing_unneeded_callbacks(
            needed, current
        )
        self.assertEqual(
            add,
            [
                {
                    "type": "project",
                    "description": "Kanban application project callback",
                    "url": None,
                    "objectId": 1,
                    "level": "owner",
                },
            ]
        )
        self.assertEqual(remove, [])

    def test_calculate_callback_needs_removing(self):
        needed = NEEDED_CALLBACKS.copy()
        current = needed.copy()
        current.insert(
            1,  # Insert not at end, to test removing from middle of list.
            {
                "type": "opportunity",
                "description": "Kanban application opportunity callback",
                "url": None,
                "objectId": 1,
                "level": "owner",
            }
        )
        add, remove = self.handler._calculate_missing_unneeded_callbacks(
            needed, current
        )
        self.assertEqual(add, [])
        self.assertEqual(
            remove,
            [
                {
                    "type": "opportunity",
                    "description": "Kanban application opportunity callback",
                    "url": None,
                    "objectId": 1,
                    "level": "owner",
                }
            ]
        )

    def test_calculate_callback_field_changed(self):
        """
        A field is different between a current and needed callback, resulting
        in one CB to add and one CB to delete.
        """
        needed = NEEDED_CALLBACKS.copy()
        current = needed.copy()
        current[0] = {
            "type": "ticket",
            "description": "wrong description!",
            "url": None,
            "objectId": 1,
            "level": "owner",
        }
        add, remove = self.handler._calculate_missing_unneeded_callbacks(
            needed, current
        )
        self.assertEqual(
            add,
            [
                {
                    "type": "ticket",
                    "description": "Kanban application ticket callback",
                    "url": None,
                    "objectId": 1,
                    "level": "owner",
                },
            ]
        )
        self.assertEqual(
            remove,
            [
                {
                    "type": "ticket",
                    "description": "wrong description!",
                    "url": None,
                    "objectId": 1,
                    "level": "owner",
                },
            ]
        )

    def test_get_callbacks(self):
        fixture = [fixtures.API_SYSTEM_CALLBACK_ENTRY]
        mocks.system_api_get_callbacks_call(fixture)

        callbacks = self.handler.get_callbacks()
        self.assertEqual(callbacks, fixture)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0045_auto_20171222_1725'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scheduleentry',
            name='name',
            field=models.CharField(null=True, max_length=250, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0033_auto_20170616_1251'),
    ]

    operations = [
        migrations.AlterField(
            model_name='opportunity',
            name='notes',
            field=models.TextField(null=True, blank=True),
        ),
    ]



# Generated by Django 3.1.4 on 2021-01-13 17:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0144_auto_20210112_1440'),
        ('djconnectwise', '0144_contactcommunication_default_flag'),
    ]

    operations = [
    ]



# Generated by Django 3.1.2 on 2020-11-30 20:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0135_auto_20201016_0810'),
    ]

    operations = [
        migrations.AddField(
            model_name='boardstatus',
            name='time_entry_not_allowed',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0046_auto_20180104_1504'),
    ]

    operations = [
        migrations.AddField(
            model_name='syncjob',
            name='sync_type',
            field=models.CharField(max_length=32, default='full'),
        ),
    ]



# Generated by Django 2.1.14 on 2019-12-20 13:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0111_auto_20191204_0838'),
    ]

    operations = [
        migrations.AddField(
            model_name='syncjob',
            name='synchronizer_class',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]



# Generated by Django 2.1.14 on 2020-09-11 16:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0122_projectphase_wbs_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.1 on 2019-07-22 15:24

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0105_auto_20190722_1444'),
    ]

    operations = [
        migrations.AddField(
            model_name='activity',
            name='agreement',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.Agreement'),
        ),
        migrations.AddField(
            model_name='activity',
            name='company',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.Company'),
        ),
    ]



# Generated by Django 2.1 on 2019-07-19 15:09

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0102_auto_20190719_1058'),
    ]

    operations = [
        migrations.CreateModel(
            name='ActivityStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(max_length=50)),
                ('default_flag', models.BooleanField(default=False)),
                ('inactive_flag', models.BooleanField(default=False)),
                ('spawn_followup_flag', models.BooleanField(default=False)),
                ('closed_flag', models.BooleanField(default=False)),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='ActivityType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(max_length=50)),
                ('points', models.IntegerField()),
                ('default_flag', models.BooleanField(default=False)),
                ('inactive_flag', models.BooleanField(default=False)),
                ('email_flag', models.BooleanField(default=False)),
                ('memo_flag', models.BooleanField(default=False)),
                ('history_flag', models.BooleanField(default=False)),
            ],
            options={
                'ordering': ('name',),
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0026_merge'),
    ]

    operations = [
        migrations.AlterField(
            model_name='opportunity',
            name='source',
            field=models.CharField(max_length=100, blank=True, null=True),
        ),
    ]



# Generated by Django 2.0.6 on 2018-06-07 12:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0060_auto_20180605_0840'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='opportunitynote',
            options={'ordering': ('-date_created', 'id'), 'verbose_name_plural': 'Opportunity notes'},
        ),
        migrations.AlterModelOptions(
            name='team',
            options={'ordering': ('name', 'id'), 'verbose_name_plural': 'Teams'},
        ),
        migrations.AlterModelOptions(
            name='timeentry',
            options={'ordering': ('-time_start', 'id'), 'verbose_name_plural': 'Time entries'},
        ),
        migrations.AddField(
            model_name='timeentry',
            name='detail_description_flag',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='timeentry',
            name='internal_analysis_flag',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='timeentry',
            name='resolution_flag',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.1.5 on 2021-03-17 14:24

from django.db import migrations, models
from django.db.migrations import RunPython


def delete_null_value_rows(apps, schema_editor):
    ContactCommunication = apps.get_model(
        'djconnectwise', 'ContactCommunication')
    queryset = ContactCommunication.objects.filter(value__isnull=True)
    queryset.delete()
    queryset = ContactCommunication.objects.filter(value__exact='')
    queryset.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0147_auto_20210223_1752'),
    ]

    operations = [
        migrations.RunPython(
            delete_null_value_rows, RunPython.noop
        ),
        migrations.AlterField(
            model_name='contactcommunication',
            name='value',
            field=models.CharField(max_length=250),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0006_auto_20170303_1242'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='boardstatus',
            options={'ordering': ('board__name', 'sort_order', 'name'), 'verbose_name_plural': 'Board statuses'},
        ),
        migrations.AlterModelOptions(
            name='connectwiseboard',
            options={'verbose_name': 'ConnectWise board', 'ordering': ('name',)},
        ),
    ]



# Generated by Django 2.2 on 2019-04-04 15:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0083_delete_callbackentry'),
    ]

    operations = [
        migrations.AlterField(
            model_name='servicenote',
            name='detail_description_flag',
            field=models.BooleanField(blank=True),
        ),
        migrations.AlterField(
            model_name='servicenote',
            name='external_flag',
            field=models.BooleanField(blank=True),
        ),
        migrations.AlterField(
            model_name='servicenote',
            name='internal_analysis_flag',
            field=models.BooleanField(blank=True),
        ),
        migrations.AlterField(
            model_name='servicenote',
            name='internal_flag',
            field=models.BooleanField(blank=True),
        ),
        migrations.AlterField(
            model_name='servicenote',
            name='resolution_flag',
            field=models.BooleanField(blank=True),
        ),
    ]



# Generated by Django 3.1.2 on 2020-12-08 11:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0137_auto_20201201_1042'),
    ]

    operations = [
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.TextField(blank=True, max_length=30, null=True)),
                ('last_name', models.TextField(blank=True, max_length=30, null=True)),
                ('address_line1', models.CharField(blank=True, max_length=250, null=True)),
                ('address_line2', models.CharField(blank=True, max_length=250, null=True)),
                ('city', models.CharField(blank=True, max_length=250, null=True)),
                ('state_identifier', models.CharField(blank=True, max_length=250, null=True)),
                ('zip', models.CharField(blank=True, max_length=250, null=True)),
                ('company', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.company')),
            ],
        ),
        migrations.CreateModel(
            name='ContactCommunication',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(blank=True, max_length=250, null=True)),
                ('contact_id', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.contact')),
            ],
        ),
        migrations.CreateModel(
            name='ContactCommunicationTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_contactcommunication',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.contactcommunication',),
        ),
        migrations.CreateModel(
            name='ContactTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_contact',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.contact',),
        ),
    ]



import os
from mock import patch

from datetime import datetime, date, time
import json
import responses

from . import fixtures
from django.utils import timezone

CW_MEMBER_IMAGE_FILENAME = 'AnonymousMember.png'


def create_mock_call(method_name, return_value, side_effect=None):
    """Utility function for mocking the specified function or method"""
    _patch = patch(method_name, side_effect=side_effect)
    mock_get_call = _patch.start()

    if not side_effect:
        mock_get_call.return_value = return_value

    return mock_get_call, _patch


def company_api_get_contacts(return_value):
    method_name = 'djconnectwise.api.CompanyAPIClient.get_contacts'
    return create_mock_call(method_name, return_value)


def company_api_get_communication_types(return_value):
    method_name = 'djconnectwise.api.CompanyAPIClient.get_communication_types'
    return create_mock_call(method_name, return_value)


def company_api_get_contact_communications(return_value):
    method_name = 'djconnectwise.api.CompanyAPIClient.' \
                  'get_contact_communications'
    return create_mock_call(method_name, return_value)


def company_info_get_company_info_call(return_value):
    method_name = 'djconnectwise.api.CompanyInfoManager.get_company_info'
    return create_mock_call(method_name, return_value)


def company_api_get_call(return_value):
    method_name = 'djconnectwise.api.CompanyAPIClient.get_companies'
    return create_mock_call(method_name, return_value)


def company_api_by_id_call(return_value, raised=None):
    method_name = 'djconnectwise.api.CompanyAPIClient.by_id'
    return create_mock_call(method_name, return_value, side_effect=raised)


def company_api_get_company_statuses_call(return_value, raised=None):
    method_name = 'djconnectwise.api.CompanyAPIClient.get_company_statuses'
    return create_mock_call(method_name, return_value, side_effect=raised)


def company_api_get_company_types_call(return_value, raised=None):
    method_name = 'djconnectwise.api.CompanyAPIClient.get_company_types'
    return create_mock_call(method_name, return_value, side_effect=raised)


def projects_api_get_project_statuses_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ProjectAPIClient.get_project_statuses'
    return create_mock_call(method_name, return_value, side_effect=raised)


def projects_api_get_project_types_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ProjectAPIClient.get_project_types'
    return create_mock_call(method_name, return_value, side_effect=raised)


def projects_api_get_project_phases_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ProjectAPIClient.get_project_phases'
    return create_mock_call(method_name, return_value, side_effect=raised)


def project_api_get_projects_call(return_value):
    method_name = 'djconnectwise.api.ProjectAPIClient.get_projects'
    return create_mock_call(method_name, return_value)


def project_api_get_project_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ProjectAPIClient.get_project'
    return create_mock_call(method_name, return_value, side_effect=raised)


def project_api_get_team_members_call(return_value):
    method_name = 'djconnectwise.api.ProjectAPIClient.get_project_team_members'
    return create_mock_call(method_name, return_value)


def _project_api_tickets_call(page=1, page_size=25, conditions=[]):
    return_value = []
    test_date = date(1948, 5, 14)
    test_time = time(12, 0, 0, tzinfo=timezone.get_current_timezone())
    test_datetime = datetime.combine(test_date, test_time)
    conditions.append('lastUpdated>' + timezone.localtime(
        value=test_datetime).isoformat()
                      )
    if page == 1:
        return_value = [fixtures.API_PROJECT_TICKET]

    return return_value


def project_api_tickets_call():
    method_name = 'djconnectwise.api.TicketAPIMixin.get_tickets'
    mock_call, _patch = create_mock_call(
        method_name,
        None,
        side_effect=_project_api_tickets_call)
    return mock_call, _patch


def project_api_tickets_test_command(return_value):
    method_name = 'djconnectwise.api.TicketAPIMixin.get_tickets'
    mock_call, _patch = create_mock_call(method_name, return_value)
    return mock_call, _patch


def sales_api_by_id_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.by_id'
    return create_mock_call(method_name, return_value, side_effect=raised)


def sales_api_get_opportunities_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.get_opportunities'
    return create_mock_call(method_name, return_value, side_effect=raised)


def sales_api_get_opportunity_statuses_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.get_opportunity_statuses'
    return create_mock_call(method_name, return_value, side_effect=raised)


def sales_api_get_opportunity_types_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.get_opportunity_types'
    return create_mock_call(method_name, return_value, side_effect=raised)


def sales_api_get_opportunity_stages_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.get_opportunity_stages'
    return create_mock_call(method_name, return_value, side_effect=raised)


def sales_api_get_sales_probabilities_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.get_probabilities'
    return create_mock_call(method_name, return_value, side_effect=raised)


def schedule_api_get_schedule_types_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ScheduleAPIClient.get_schedule_types'
    return create_mock_call(method_name, return_value, side_effect=raised)


def schedule_api_get_schedule_statuses_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ScheduleAPIClient.get_schedule_statuses'
    return create_mock_call(method_name, return_value, side_effect=raised)


def schedule_api_get_schedule_entries_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ScheduleAPIClient.get_schedule_entries'
    return create_mock_call(method_name, return_value, side_effect=raised)


def schedule_api_get_schedule_entry_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ScheduleAPIClient.get_schedule_entry'
    return create_mock_call(method_name, return_value, side_effect=raised)


def schedule_api_get_calendars_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ScheduleAPIClient.get_calendars'
    return create_mock_call(method_name, return_value, side_effect=raised)


def schedule_api_get_holidays_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ScheduleAPIClient.get_holidays'
    return create_mock_call(method_name, return_value, side_effect=raised)


def schedule_api_get_holiday_lists_call(return_value, raised=None):
    method_name = 'djconnectwise.api.ScheduleAPIClient.get_holiday_lists'
    return create_mock_call(method_name, return_value, side_effect=raised)


def time_api_get_time_entries_call(return_value, raised=None):
    method_name = 'djconnectwise.api.TimeAPIClient.get_time_entries'
    return create_mock_call(method_name, return_value, side_effect=raised)


def sales_api_get_activities_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.get_activities'
    return create_mock_call(method_name, return_value, side_effect=raised)


def sales_api_get_activities_statuses_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.get_activity_statuses'
    return create_mock_call(method_name, return_value, side_effect=raised)


def sales_api_get_activities_types_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.get_activity_types'
    return create_mock_call(method_name, return_value, side_effect=raised)


def sales_api_get_single_activity_call(return_value, raised=None):
    method_name = 'djconnectwise.api.SalesAPIClient.get_single_activity'
    return create_mock_call(method_name, return_value, side_effect=raised)


def _service_api_tickets_call(page=1, page_size=25, conditions=[]):
    return_value = []
    test_date = date(1948, 5, 14)
    test_time = time(12, 0, 0, tzinfo=timezone.get_current_timezone())
    test_datetime = datetime.combine(test_date, test_time)
    conditions.append('lastUpdated>' + timezone.localtime(
        value=test_datetime).isoformat()
        )
    if page == 1:
        return_value = [fixtures.API_SERVICE_TICKET]
    return return_value


def service_api_tickets_call():
    method_name = 'djconnectwise.api.TicketAPIMixin.get_tickets'
    mock_call, _patch = create_mock_call(
        method_name,
        None,
        side_effect=_service_api_tickets_call)
    return mock_call, _patch


def _service_api_get_ticket_call(ticket_id):
    return fixtures.API_SERVICE_TICKET_MAP.get(ticket_id)


def service_api_get_ticket_call(raised=None):
    method_name = 'djconnectwise.api.TicketAPIMixin.get_ticket'
    mock_call, _patch = create_mock_call(
        method_name,
        None,
        side_effect=raised if raised else _service_api_get_ticket_call)
    return mock_call, _patch


def service_api_get_boards_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_boards'
    return create_mock_call(method_name, return_value)


def service_api_update_ticket_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.update_ticket'
    return create_mock_call(method_name, return_value)


def service_api_get_statuses_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_statuses'
    return create_mock_call(method_name, return_value)


def service_api_get_priorities_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_priorities'
    return create_mock_call(method_name, return_value)


def service_api_get_teams_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_teams'
    return create_mock_call(method_name, return_value)


def service_api_get_notes_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_notes'
    return create_mock_call(method_name, return_value)


def service_api_get_slas_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_slas'
    return create_mock_call(method_name, return_value)


def service_api_get_sla_priorities_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_slapriorities'
    return create_mock_call(method_name, return_value)


def service_api_get_types_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_types'
    return create_mock_call(method_name, return_value)


def service_api_get_subtypes_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_subtypes'
    return create_mock_call(method_name, return_value)


def service_api_get_items_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_items'
    return create_mock_call(method_name, return_value)


def sales_api_get_opportunity_notes_call(return_value):
    method_name = 'djconnectwise.api.SalesAPIClient.get_notes'
    return create_mock_call(method_name, return_value)


def service_api_get_locations_call(return_value):
    method_name = 'djconnectwise.api.ServiceAPIClient.get_locations'
    return create_mock_call(method_name, return_value)


def system_api_get_connectwise_version_call(return_value):
    method_name = 'djconnectwise.api.SystemAPIClient.get_connectwise_version'
    return create_mock_call(method_name, return_value)


def system_api_get_members_call(return_value):
    method_name = 'djconnectwise.api.SystemAPIClient.get_members'
    return create_mock_call(method_name, return_value)


def system_api_get_member_image_by_photo_id_call(return_value):
    method_name = 'djconnectwise.api.SystemAPIClient.' \
                  + 'get_member_image_by_photo_id'

    return create_mock_call(method_name, return_value)


def system_api_get_member_count_call(return_value):
    method_name = 'djconnectwise.api.SystemAPIClient.get_members'
    return create_mock_call(method_name, return_value)


def system_api_create_callback_call(return_value):
    method_name = 'djconnectwise.api.SystemAPIClient.create_callback'
    return create_mock_call(method_name, return_value)


def system_api_delete_callback_call(return_value):
    method_name = 'djconnectwise.api.SystemAPIClient.delete_callback'
    return create_mock_call(method_name, return_value)


def system_api_get_callbacks_call(return_value):
    method_name = 'djconnectwise.api.SystemAPIClient.get_callbacks'
    return create_mock_call(method_name, return_value)


def system_api_get_territories_call(return_value):
    method_name = 'djconnectwise.api.SystemAPIClient.get_territories'
    return create_mock_call(method_name, return_value)


def system_api_get_other_call(return_value):
    method_name = 'djconnectwise.api.SystemAPIClient.get_mycompanyother'
    return create_mock_call(method_name, return_value)


def cw_api_fetch_resource_call(return_value):
    method_name = 'djconnectwise.api.ConnectWiseAPIClient.fetch_resource'
    return create_mock_call(method_name, return_value)


def get(url, data, headers=None, status=200):
    """Set up requests mock for given URL and JSON-serializable data."""
    get_raw(url, json.dumps(data), "application/json", headers, status=status)


def time_api_get_work_types_call(return_value):
    method_name = 'djconnectwise.api.TimeAPIClient.get_work_types'
    return create_mock_call(method_name, return_value)


def time_api_get_work_roles_call(return_value):
    method_name = 'djconnectwise.api.TimeAPIClient.get_work_roles'
    return create_mock_call(method_name, return_value)


def finance_api_get_agreements_call(return_value):
    method_name = 'djconnectwise.api.FinanceAPIClient.get_agreements'
    return create_mock_call(method_name, return_value)


def get_raw(url, data, content_type="application/octet-stream", headers=None,
            status=200):
    """Set up requests mock for given URL."""
    responses.add(
        responses.GET,
        url,
        body=data,
        status=status,
        content_type=content_type,
        adding_headers=headers,
    )


def get_member_avatar():
    """Return the avatar image data in the tests directory."""
    cw_member_image_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        CW_MEMBER_IMAGE_FILENAME
    )
    with open(cw_member_image_path, 'rb') as anonymous_image_file:
        return anonymous_image_file.read()
